#!/usr/bin/python
import sys
import os

# Globals

defPLPath = os.path.expanduser("~/Library/Preferences/com.nakiwo.Doukutsu.plist")
strings = {'getPL':"Enter the full path of the property list file.\nOr type D to use the default or Q to cancel.",
           'noPrefs':"Sorry, but the prefs file cannot be found.\nPerhaps Cave Story has not been installed or run on this account.",
           'notMac':"Sorry, the default path is only available on the Mac."}

def parseInput():
    """Get command-line input"""
    currentArg = ""
    tasks = []
    if len(sys.argv) == 1:
        tasks.append( ("interactive",) )
    i = 1
    while i < len(sys.argv):
        if sys.argv[i] == '-?' or sys.argv[i] == '-h':
            displayUsage()
            sys.exit()
        if sys.argv[i] == '-?x' or sys.argv[i] == '-hx':
            advancedHelp()
            sys.exit()
        elif sys.argv[i] == '-e':
            x = ""
            try:
                if sys.argv[i+1] == '@': x = defPLPath
                else: x = sys.argv[i+1]
                tasks.append( ("extract-mac-profile",x,sys.argv[i+2]) )
            except:
                print "Error: Source or destination not specified for extraction."
                sys.exit()
            i += 3
        elif sys.argv[i] == '-s':
            try:
                if sys.argv[i+2] == '@': x = defPLPath
                else: x = sys.argv[i+2]
                tasks.append( ("store-mac-profile",sys.argv[i+1],x) )
            except:
                print "Error: Source or destination not specified for storing profile."
                sys.exit()
            i += 3
        elif sys.argv[i] == '-ec':
            try:
                if sys.argv[i+1] == '@': x = defPLPath
                else: x = sys.argv[i+1]
                tasks.append( ("extract-mac-profile",x,sys.argv[i+2]) )
                tasks.append( ("convert-profile",sys.argv[i+2],sys.argv[i+2]) )
            except:
                print "Error: Source or destination not specified for extraction."
                sys.exit()
            i += 3
        elif sys.argv[i] == '-cs':
            try:
                if sys.argv[i+2] == '@': x = defPLPath
                else: x = sys.argv[i+2]
                tasks.append( ("convert-profile",sys.argv[i+1],".pycs_temp") )
                tasks.append( ("store-mac-profile",".pycs_temp",x) )
                tasks.append( ("clean-temp-file",".pycs_temp") )
            except:
                print "Error: Source or destination not specified for storing profile."
                sys.exit()
            i += 3
        elif sys.argv[i][:2] == '-w':
            arg = sys.argv[i][2:]
            tasks.append( ("remove-weapon", int(arg), sys.argv[i+1]) )
            i += 1
        elif sys.argv[i][:2] == '+w':
            # Syntax: +w4=12,100 : weapon=energy,maxammo
            args = sys.argv[i][2:]
            try:
                args = args.split('=')
                args[1] = args[1].split(',')
                tasks.append( ("add-weapon", int(args[0]), int(args[1]), int(args[2]), sys.argv[i+1]) )
            except:
                print "Syntax error in remove weapon directive."
                sys.exit()
            i += 2
        elif sys.argv[i][:2] == '-i':
            arg = sys.argv[i][2:]
            tasks.append( ("remove-item", int(arg), sys.argv[i+1]) )
            i += 2
        elif sys.argv[i][:2] == '+i':
            arg = sys.argv[i][2:]
            tasks.append( ("add-item", int(arg), sys.argv[i+1]) )
            i += 2
        elif sys.argv[i][:2] == '-f':
            arg = sys.argv[i][2:]
            task.append( ("unset-flag",) )
        elif sys.argv[i][:2] == '+f':
            arg = sys.argv[i][2:]
            task.append( ("set-flag",) )
    return tasks
#    if len(sys.argv) > 1:
#        if sys.argv[1] == "-?" or sys.argv[1] == "-h" or sys.argv[1] == "-h":
#            print "Usage:", sys.argv[0], "name [opt] [source [dest]]"
#            sys.exit()
#        elif sys.argv[1] == "-e":
#            mode = "extract-mac-profile"
#            source = sys.argv[2] if len(sys.argv) > 2 else "com.nakiwo.doukutsu.plist"
#            if sys.platform == "darwin" and '/' not in source and '\\' not in source:
#                source = "~/Library/Preferences/" + source;
#            dest = sys.argv[3] if len(sys.argv) > 3 else "profile.dsv"
#        elif sys.argv[1] == "-s":
#            mode = "store-mac-profile"
#            source = sys.argv[2] if len(sys.argv) > 2 else "profile.dsv"
#            dest = sys.argv[3] if len(sys.argv) > 3 else defPLPath
#        elif sys.argv[1] == "-m":
#            mode = "win-to-mac"
#            source = sys.argv[2] if len(sys.argv) > 2 else "profile.dat"
#            dest = sys.argv[3] if len(sys.argv) > 3 else "profile.dsv"
#        elif sys.argv[1] == "-w":
#            mode = "mac-to-win"
#            source = sys.argv[2] if len(sys.argv) > 2 else "profile.dsv"
#            dest = sys.argv[3] if len(sys.argv) > 3 else "profile.dat"
#        else:
#            source = sys.argv[2]
#            if len(sys.argv) > 3: dest = sys.argv[3]
#    return mode

def getUserInput():
    """Get user input"""
    print "Which operation would you like to perform?"
    print "E - Extract a save from a Macintosh property list and save it to a file."
    print "S - Store a save in a Macintosh property list, overwriting the existing one."
    print "C - Convert a save file (if it's Windows format it will become Mac format and vice verse)"
    print "X - Edit a save file."
    print "Q - Cancel operation."
    input = sys.stdin.read(1); sys.stdin.readline()
    if input.upper() == "E":
        mode = "extract-mac-profile"
    elif input.upper() == 'S':
        mode = "store-mac-profile"
    elif input.upper() == 'C':
        mode = "convert-profile"
    elif input.upper() == 'X':
        print "Save-file editing not yet implemented."
        interactiveEditMode()
    elif input.upper() == 'Q':
        sys.exit()
    else: mode = ""
    if mode != "":
        source = getSourceFile(mode)
        dest = getDestFile(mode)
        functions[mode](source,dest)
    getUserInput()

def interactiveEditMode():
    print "Save-file editing not yet implemented."

def main():
    tasks = parseInput()
    #print tasks
    for x in tasks:
        functions[x[0]](*x[1:])

def displayUsage():
    print "Usage:", sys.argv[0], "[opt [params]]*"
    print
    print "Available options are -e, -s, -c, -w, +w, -i, +i, -f, +f, -x:"
    print "-e prefFile dest"
    print "    Extracts a Mac profile from the property list into a file."
    print "    Use @ for the prefFile to indicate the default property list."
    print "-s source prefFile"
    print "    Stores a Mac profile in the property list, replacing the one already there."
    print "    Use @ to for the prefFile indicate the default property list."
    print "-c source dest"
    print "    Converts a Mac profile to a Windows profilem, or a Windows profile to a Mac profile."
    print "+wx=y,z"
    print "    Adds weapon x to your inventory with y experience and z max ammo."
    print "-wx"
    print "    Removes weapon x from your inventory."
    print "+ix"
    print "    Adds item x to your inventory."
    print "-ix"
    print "    Removes item x from your inventory."
    print "+fx"
    print "    Sets flag x."
    print "-fx"
    print "    Unsets flag x."
    print "-x"
    print "    Advanced editing. For more information on advanced edits, specify -hx or -?x in the command line."
    print "-h"
    print "    Show this help."
    
def advancedHelp():
    print "Advanced edits are not yet implemented."

def getSourceFile(mode):
    source = ""
    while source == "":
        #print "--> Fetching source..."
        if mode == "extract-mac-profile":
            print strings['getPL']
        elif mode == "store-mac-profile":
            print "Enter the filename of the profile to store:"
            print "Or type Q to cancel."
        else:
            print "Enter the save file to convert:"
            print "Or type Q to cancel."
        input = sys.stdin.readline()[:-1]
        if input.upper() == 'Q':
            sys.exit()
        elif input.upper() == 'D' and mode == "extract-mac-profile":
            if sys.platform == "darwin":
                if not os.path.exists(defPLPath):
                    print strings['noPrefs']
                else:
                    source = defPLPath
            else:
                print strings['notMac']
        else:
            if not os.path.exists(input):
                print "File", input, "not found."
            else: source = input
    return source

def getDestFile(mode):
    dest = ""
    while dest == "":
        #print "--> Fetching dest..."
        if mode == "extract-mac-profile":
            print "Enter the filename to extract the profile to:"
            print "Or type Q to cancel."
        elif mode == "store-mac-profile":
            print strings['getPL']
        else:
            print "Enter the file to save the converted profile to:"
            print "Or type Q to cancel."
        input = sys.stdin.readline()[:-1]
        if input.upper() == 'Q':
            sys.exit()
        elif input.upper() == 'D' and mode == "store-mac-profile":
            if sys.platform == "darwin":
                if not os.path.exists(defPLPath):
                    print strings['noPrefs']
                else:
                    dest = defPLPath
            else:
                print strings['notMac']
        else: dest = input
    return dest

def extract(source, dest):
    import plistlib as plist
    import subprocess as fork
    prefs = fork.Popen(['plutil','-convert','xml1','-o','-',source], stdout=fork.PIPE).communicate()[0]
    prefs = plist.readPlistFromString(prefs)
    data = prefs['Profile.dat']
    profile = file(dest,'wb')
    profile.write(data.data)
    profile.close()
    print "--> Profile extracted to", dest

def store(source, dest):
    import plistlib as plist
    import subprocess as fork
    #print dest
    prefs = fork.Popen(['plutil','-convert','xml1','-o','-',dest], stdout=fork.PIPE).communicate()[0]
    prefs = plist.readPlistFromString(prefs)
    profile = file(source,'rb')
    data = profile.read()
    prefs['Profile.dat'].data = data
    profile.close()
    prefsFile = file(dest,'wb')
    plist.writePlist(prefs,dest)
    print "--> Profile from", source, "has been stored"

def convert(source, dest):
    orig = file(source,'rb')
    new = file(dest,'wb')
    def copy(n):
        new.write(orig.read(n))
    def swap(n):
        temp = orig.read(n)
        temp2 = []
        for c in temp: temp2 += c
        temp2.reverse()
        temp = ''.join(temp2)
        new.write(temp)
    copy(8)   # 000-007 | header or something
    swap(4)   # 008-00B | current map
    swap(4)   # 00C-00F | current song?
    swap(4)   # 010-013 | horizontal pos
    swap(4)   # 014-017 | vertical pos
    swap(4)   # 018-01B | facing direction
    swap(2)   # 01C-01D | max HP
    swap(2)   # 01E-01F | whimsical star
    swap(2)   # 020-021 | Current HP
    copy(2)   # 022-023 | Unused? Possibly alignment filler
    swap(4)   # 024-027 | Current Weapon
    copy(4)   # 028-02B | Unused?
    swap(4)   # 02C-02F | Equipped Items
    copy(4)   # 030-033 | Unused?
    swap(4)   # 034-037 | Time
    swap(4)   # 038-03B | Slot 1 Weapon Type
    swap(4)   # 03C-03F | Slot 1 Weapon Level
    swap(4)   # 040-043 | Slot 1 Weapon Energy
    swap(4)   # 044-047 | Slot 1 Weapon Max Ammo
    swap(4)   # 048-04B | Slot 1 Weapon Current Ammo
    swap(4)   # 04C-04F | Slot 2 Weapon Type
    swap(4)   # 050-053 | Slot 2 Weapon Level
    swap(4)   # 054-057 | Slot 2 Weapon Energy
    swap(4)   # 058-05B | Slot 2 Weapon Max Ammo
    swap(4)   # 05C-05F | Slot 2 Weapon Current Ammo
    swap(4)   # 060-063 | Slot 3 Weapon Type
    swap(4)   # 064-067 | Slot 3 Weapon Level
    swap(4)   # 068-06B | Slot 3 Weapon Energy
    swap(4)   # 06C-06F | Slot 3 Weapon Max Ammo
    swap(4)   # 070-073 | Slot 3 Weapon Current Ammo
    swap(4)   # 074-077 | Slot 4 Weapon Type
    swap(4)   # 078-07B | Slot 4 Weapon Level
    swap(4)   # 07C-07F | Slot 4 Weapon Energy
    swap(4)   # 080-083 | Slot 4 Weapon Max Ammo
    swap(4)   # 084-087 | Slot 4 Weapon Current Ammo
    swap(4)   # 088-08B | Slot 5 Weapon Type
    swap(4)   # 08C-08F | Slot 5 Weapon Level
    swap(4)   # 090-093 | Slot 5 Weapon Energy
    swap(4)   # 094-097 | Slot 5 Weapon Max Ammo
    swap(4)   # 098-09B | Slot 5 Weapon Current Ammo
    swap(4)   # 09C-09F | Slot 6 Weapon Type
    swap(4)   # 0A0-0A3 | Slot 6 Weapon Level
    swap(4)   # 0A4-0A7 | Slot 6 Weapon Energy
    swap(4)   # 0A8-0AB | Slot 6 Weapon Max Ammo
    swap(4)   # 0AC-0AF | Slot 6 Weapon Current Ammo
    swap(4)   # 0B0-0B3 | Slot 7 Weapon Type
    swap(4)   # 0B4-0B7 | Slot 7 Weapon Level
    swap(4)   # 0B8-0BB | Slot 7 Weapon Energy
    swap(4)   # 0BC-0BF | Slot 7 Weapon Max Ammo
    swap(4)   # 0C0-0C3 | Slot 7 Weapon Current Ammo
    swap(4)   # 0C4-0C7 | Slot 8 Weapon Type
    swap(4)   # 0C8-0CB | Slot 8 Weapon Level
    swap(4)   # 0CC-0CF | Slot 8 Weapon Energy
    swap(4)   # 0D0-0D3 | Slot 8 Weapon Max Ammo
    swap(4)   # 0D4-0D7 | Slot 8 Weapon Current Ammo
    swap(4)   # 0D8-0DB | Inventory R1C1
    swap(4)   # 0DC-0DF | Inventory R1C2
    swap(4)   # 0E0-0E3 | Inventory R1C3
    swap(4)   # 0E4-0E7 | Inventory R1C4
    swap(4)   # 0E8-0EB | Inventory R1C5
    swap(4)   # 0EC-0EF | Inventory R1C6
    swap(4)   # 0F0-0F3 | Inventory R2C1
    swap(4)   # 0F4-0F7 | Inventory R2C2
    swap(4)   # 0F8-0FB | Inventory R2C3
    swap(4)   # 0FC-0FF | Inventory R2C4
    swap(4)   # 100-103 | Inventory R2C5
    swap(4)   # 104-107 | Inventory R2C6
    swap(4)   # 108-10B | Inventory R3C1
    swap(4)   # 10C-10F | Inventory R3C2
    swap(4)   # 110-113 | Inventory R3C3
    swap(4)   # 114-117 | Inventory R3C4
    swap(4)   # 118-11B | Inventory R3C5
    swap(4)   # 11C-11F | Inventory R3C6
    swap(4)   # 120-123 | Inventory R4C1
    swap(4)   # 124-127 | Inventory R4C2
    swap(4)   # 128-12B | Inventory R4C3
    swap(4)   # 12C-12F | Inventory R4C4
    swap(4)   # 130-133 | Inventory R4C5
    swap(4)   # 134-137 | Inventory R4C6
    swap(4)   # 138-13B | Inventory R5C1
    swap(4)   # 13C-13F | Inventory R5C2
    swap(4)   # 140-143 | Inventory R5C3
    swap(4)   # 144-147 | Inventory R5C4
    swap(4)   # 148-14B | Inventory R5C5
    swap(4)   # 14C-14F | Inventory R5C6
    swap(4)   # 150-153 | Inventory R6C1
    swap(4)   # 154-157 | Inventory R6C2
    swap(4)   # 158-15B | Slot 1 Teleporter Menu
    swap(4)   # 15C-15F | Slot 1 Teleporter Location
    swap(4)   # 160-163 | Slot 2 Teleporter Menu
    swap(4)   # 164-167 | Slot 2 Teleporter Location
    swap(4)   # 168-16B | Slot 3 Teleporter Menu
    swap(4)   # 16C-16F | Slot 3 Teleporter Location
    swap(4)   # 170-173 | Slot 4 Teleporter Menu
    swap(4)   # 174-177 | Slot 4 Teleporter Location
    swap(4)   # 178-17B | Slot 5 Teleporter Menu
    swap(4)   # 17C-17F | Slot 5 Teleporter Location
    swap(4)   # 180-183 | Slot 6 Teleporter Menu
    swap(4)   # 184-187 | Slot 6 Teleporter Location
    swap(4)   # 188-18B | Slot 7 Teleporter Menu
    swap(4)   # 18C-18F | Slot 7 Teleporter Location
    swap(4)   # 190-193 | Slot 8 Teleporter Menu
    swap(4)   # 194-197 | Slot 8 Teleporter Location
    copy(128) # 198-217 | Unused?
    copy(4)   # 218-21B | "FLAG"
    copy(232) # 21C-303 | Flags - don't swap
    # And the remainder of the file:
    new.write(orig.read())
    orig.close()
    new.close()
    print "--> Profile converted."

def deleteTemp(fileName):
    print "Removing temporary file", fileName + ',', "please confirm."
    if raw_input().upper() == 'Y':
        os.remove(fileName)
    print "--> Temporary file removed."

functions = {"extract-mac-profile":extract, "store-mac-profile":store, "convert-profile":convert,
             "remove-weapon":None, "add-weapon":None, "remove-item":None, "add-item":None,
             "set-flag":None, "unset-flag":None, "advanced-edit":None, "interactive":getUserInput,
             "clean-temp-file":deleteTemp}

main()
